# Generated by Django 2.1.5 on 2020-02-21 13:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='device',
            name='manufacturer',
            field=models.CharField(blank=True, max_length=300, null=True),
        ),
        migrations.AddField(
            model_name='device',
            name='uuid',
            field=models.CharField(blank=True, max_length=300, null=True),
        ),
        migrations.AddField(
            model_name='device',
            name='version',
            field=models.CharField(blank=True, max_length=300, null=True),
        ),
        migrations.AlterField(
            model_name='device',
            name='token',
            field=models.CharField(blank=True, db_index=True, max_length=300, null=True),
        ),
    ]
